from typing import Optional, List, Literal
from pylume import PyLume, VMRunOpts, SharedDirectory
import asyncio
from .models import Computer as ComputerConfig, Image, Display
from .interface.factory import InterfaceFactory
from pylume import ImageRef, VMUpdateOpts, PyLume
import time

OSType = Literal['macos', 'linux']

class Computer:
    def __init__(
        self,
        display: Display,
        memory: str,
        cpu: str,
        os: OSType = 'macos',
        name: str = '',
        image: str = "macos-sequoia-xcode:latest",
        shared_directories: Optional[List[SharedDirectory]] = None,
        use_host_computer_server: bool = False,
    ):
        """Initialize a new Computer instance.
        
        Args:
            image: The VM image name
            tag: The VM image tag
            name: The VM name
            display: The display configuration
            memory: The VM memory allocation
            cpu: The VM CPU allocation
            os: The operating system type ('macos' or 'linux')
            shared_directories: Optional list of directories to share with the VM
            use_host_computer_server: If True, target localhost instead of starting a VM
        """
        if not use_host_computer_server:
            if ':' not in image or len(image.split(':')) != 2:
                raise ValueError("Image must be in the format <image_name>:<tag>")
            
            if not name:
                # Normalize the name to be used for the VM
                name = image.replace(':', '_')
            
            self.config = ComputerConfig(
                image=image.split(':')[0],
                tag=image.split(':')[1],
                name=name,
                display=display,
                memory=memory,
                cpu=cpu,
            )

        self._interface = None
        self.os = os
        self.shared_directories = shared_directories or []
        self._pylume_context = None
        self.use_host_computer_server = use_host_computer_server
        
    async def __aenter__(self):
        """Enter the async context manager."""
        if not self.use_host_computer_server:
            self.config.pylume = PyLume(debug=True, port=3000, use_existing_server=False)
            self._pylume_context = await self.config.pylume.__aenter__()
        return self
        
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        """Exit the async context manager."""
        if self._pylume_context:
            await self.config.pylume.__aexit__(exc_type, exc_val, exc_tb)
            
    @property
    def ip(self) -> str:
        """Get the IP address of the VM or localhost if using host computer server."""
        if self.use_host_computer_server:
            return "127.0.0.1"
        return self.config.ip
        
    async def wait_vm_ready(self, timeout: int = 60, interval: float = 1.0) -> None:
        """Wait for the VM to be ready with required properties (ip_address, vnc_address).
        
        Args:
            timeout: Maximum time to wait in seconds
            interval: Time between attempts in seconds
            
        Raises:
            TimeoutError: If the VM is not ready within the timeout period
        """
        if self.use_host_computer_server:
            return None
            
        start_time = time.time()
        while time.time() - start_time < timeout:
            try:
                vm = await self.config.pylume.get_vm(self.config.name)
                if vm.ip_address and vm.vnc_url:
                    return vm
            except Exception:
                pass
            await asyncio.sleep(interval)
                
        raise TimeoutError(f"VM {self.config.name} not ready after {timeout} seconds")

    async def run(self, no_display: bool = False):
        """Run the VM and initialize the computer interface."""
        if not self._pylume_context and not self.use_host_computer_server:
            raise RuntimeError("Computer must be used as an async context manager when not using host computer server")
        
        ip_address = "127.0.0.1"

        if not self.use_host_computer_server:
            # Try to get the VM, if it doesn't exist, create it and pull the image
            try:
                vm = await self.config.pylume.get_vm(self.config.name)
            except Exception:
                image_ref = ImageRef(
                    image=self.config.image,
                    tag=self.config.tag,
                    registry="ghcr.io",
                    organization="trycua"
                )
                await self.config.pylume.pull_image(image_ref, name=self.config.name)

            # Run with shared directory
            run_opts = VMRunOpts(
                no_display=False,
                shared_directories=self.shared_directories
            )
            await self.config.pylume.run_vm(self.config.name, run_opts)
            
            # Wait for VM to be ready with required properties
            vm = await self.wait_vm_ready()
            ip_address = vm.ip_address
        
        # Initialize the interface using the factory with the specified OS
        self._interface = InterfaceFactory.create_interface_for_os(
            os=self.os,
            ip_address=ip_address
        )

        # wait for the websocket interface to be ready
        await self._interface.wait_for_ready()

        # Create an event to keep the VM running in background if needed
        if not self.use_host_computer_server:
            self._stop_event = asyncio.Event()
            self._keep_alive_task = asyncio.create_task(self._stop_event.wait())

    async def stop(self):
        """Stop the VM and close the interface."""
        if hasattr(self, '_stop_event'):
            self._stop_event.set()
            if hasattr(self, '_keep_alive_task'):
                await self._keep_alive_task
        if self._interface:
            self._interface.close()
        await self.config.pylume.stop_vm(self.config.image.name)
    
    async def update(self, cpu: Optional[int] = None, memory: Optional[str] = None):
        """Update VM settings."""
        update_opts = VMUpdateOpts(
            cpu=cpu or int(self.config.cpu),
            memory=memory or self.config.memory
        )
        await self.config.pylume.update_vm(self.config.image.name, update_opts)
        
    @property
    def interface(self):
        """Get the computer interface for interacting with the VM.
        
        Returns:
            BaseComputerInterface: The interface for controlling the VM
            
        Raises:
            RuntimeError: If the interface is not initialized (run() not called)
        """
        if self._interface is None:
            raise RuntimeError("Computer interface not initialized. Call run() first.")
        return self._interface

    def __getattr__(self, name: str):
        """Delegate all other method calls to the interface.
        
        This is kept for backward compatibility, prefer using computer.interface directly.
        """
        return getattr(self.interface, name)
