from abc import ABC, abstractmethod
from typing import Optional, Dict, Any, Tuple

class BaseComputerInterface(ABC):
    """Abstract base class for OS-specific computer interfaces.
    
    This class defines the contract that all OS-specific implementations must follow.
    
    Categories:
    - Connection Management: Methods for managing connections to the remote system
    - Mouse Actions: Methods for controlling the mouse
    - Keyboard Actions: Methods for keyboard input
    - Scrolling Actions: Methods for scrolling
    - Screen Actions: Methods for screen interaction and information
    - Clipboard Actions: Methods for clipboard operations
    - File System Actions: Methods for file system operations
    - Accessibility Actions: Methods for accessibility features
    """

    def __init__(self, ip_address: str, username: str = "lume", password: str = "lume"):
        self.ip_address = ip_address
        self.username = username
        self.password = password
        
    @abstractmethod
    async def wait_for_ready(self, timeout: int = 60, interval: float = 1.0) -> None:
        """Wait for the interface to be ready."""
        pass
        
    @abstractmethod
    async def close(self) -> None:
        """Close all connections."""
        pass

    # Mouse Actions
    @abstractmethod
    async def left_click(self, x: Optional[int] = None, y: Optional[int] = None) -> None:
        """Perform a left click."""
        pass

    @abstractmethod
    async def right_click(self, x: Optional[int] = None, y: Optional[int] = None) -> None:
        """Perform a right click."""
        pass

    @abstractmethod
    async def double_click(self, x: Optional[int] = None, y: Optional[int] = None) -> None:
        """Perform a double click."""
        pass

    @abstractmethod
    async def move_cursor(self, x: int, y: int) -> None:
        """Move the cursor to specified position."""
        pass

    @abstractmethod
    async def drag_to(self, x: int, y: int, button: str = "left", duration: float = 0.5) -> None:
        """Drag from current position to specified coordinates.
        
        Args:
            x: The x coordinate to drag to
            y: The y coordinate to drag to
            button: The mouse button to use ('left', 'middle', 'right')
            duration: How long the drag should take in seconds
        """
        pass

    # Keyboard Actions
    @abstractmethod
    async def type(self, text: str) -> None:
        """Type the specified text."""
        pass

    @abstractmethod
    async def press_key(self, key: str) -> None:
        """Press a single key."""
        pass

    @abstractmethod
    async def hotkey(self, *keys: str) -> None:
        """Press multiple keys simultaneously."""
        pass

    # Scrolling Actions
    @abstractmethod
    async def scroll_down(self, clicks: int = 1) -> None:
        """Scroll down."""
        pass

    @abstractmethod
    async def scroll_up(self, clicks: int = 1) -> None:
        """Scroll up."""
        pass

    # Screen Actions
    @abstractmethod
    async def screenshot(self) -> bytes:
        """Take a screenshot."""
        pass

    @abstractmethod
    async def get_screen_size(self) -> Dict[str, int]:
        """Get screen dimensions."""
        pass

    @abstractmethod
    async def get_cursor_position(self) -> Dict[str, int]:
        """Get current cursor position."""
        pass

    # Clipboard Actions
    @abstractmethod
    async def copy_to_clipboard(self) -> str:
        """Get clipboard content."""
        pass

    @abstractmethod
    async def set_clipboard(self, text: str) -> None:
        """Set clipboard content."""
        pass

    # File System Actions
    @abstractmethod
    async def file_exists(self, path: str) -> bool:
        """Check if file exists."""
        pass

    @abstractmethod
    async def directory_exists(self, path: str) -> bool:
        """Check if directory exists."""
        pass

    @abstractmethod
    async def run_command(self, command: str) -> Tuple[str, str]:
        """Run shell command."""
        pass

    # Accessibility Actions
    @abstractmethod
    async def get_accessibility_tree(self) -> Dict:
        """Get accessibility tree."""
        pass

    @abstractmethod
    async def find_element_by_criteria(self, 
                                     role: Optional[str] = None, 
                                     title: Optional[str] = None,
                                     value: Optional[str] = None) -> Optional[Dict]:
        """Find element by accessibility criteria."""
        pass 