import platform
from typing import Type, Literal
from .base import BaseComputerInterface
from .macos import MacOSComputerInterface
from .linux import LinuxComputerInterface

OSType = Literal['macos', 'linux']

class InterfaceFactory:
    """Factory for creating OS-specific computer interfaces."""
    
    @staticmethod
    def create_interface_for_os(os: OSType, ip_address: str, username: str = "lume", password: str = "lume") -> BaseComputerInterface:
        """Create and return appropriate interface for the specified OS.
        
        Args:
            os: The operating system type ('macos' or 'linux')
            ip_address: The IP address of the remote computer
            username: The username for connection
            password: The password for connection
            
        Returns:
            BaseComputerInterface: The appropriate interface implementation for the OS
            
        Raises:
            NotImplementedError: If the OS is not supported
        """
        if os == 'macos':
            return MacOSComputerInterface(ip_address, username, password)
        elif os == 'linux':
            return LinuxComputerInterface(ip_address, username, password)
        else:
            raise NotImplementedError(f"OS '{os}' is not supported") 